#
# Copyright 2015-2018 Andrey Galkin <andrey@futoin.org>
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from ..runtimetool import RuntimeTool


class exeTool(RuntimeTool):
    """Dummy tool to execute files directly"""
    __slots__ = ()

    def initEnv(self, env):
        self._have_tool = True

    def onRun(self, config, svc, args):
        env = config['env']
        self._executil.callInteractive([svc['path']] + args)
